@@ -4,12 +4,26 @@ expect_snapshot_trace <- function(trace,
   expect_snapshot({
     "Full"
     print(trace, simplify = "none", dir = dir, srcrefs = srcrefs)
-    "Collapsed"
-    print(trace, simplify = "collapse", dir = dir, srcrefs = srcrefs)
+    "Focused"
+    print_focused_trace(trace, dir = dir, srcrefs = srcrefs)
     "Branch"
     print(trace, simplify = "branch", dir = dir, srcrefs = srcrefs)
   })
 }
+print_focused_trace <- function(trace, ...) {
+  with_options(
+    "rlang:::trace_deemph" = function(x) sprintf("<<%s>>", x),
+    print(trace, ..., simplify = "none", drop = TRUE)
+  )
+}
+
+print_highlighted_trace <- function(x, ...) {
+  local_options(
+    "rlang_trace_format_srcrefs" = FALSE,
+    "rlang:::trace_test_highlight" = TRUE
+  )
+  print(x, ..., simplify = "none", drop = TRUE)
+}
 
 expect_trace_length <- function(x, n) {
   expect_equal(trace_length(x), n)
@@ -20,7 +34,7 @@ expect_equal_trace <- function(x, y) {
   expect_equal(x$calls, y$calls)
 }
 
-render_md <- function(file) {
+render_md <- function(file, env = global_env()) {
   skip_if_not_installed("rmarkdown")
   skip_if_not_installed("knitr")
 
@@ -32,7 +46,7 @@ render_md <- function(file) {
     output_format = "md_document",
     output_file = out_file,
     quiet = TRUE,
-    envir = globalenv()
+    envir = env
   )
 
   readLines(out_file)
