@@ -1,5 +1,3 @@
-context("s3")
-
 test_that("inherits from all classes", {
   x <- structure(list(), class = c("foo", "bar", "baz"))
 
@@ -69,7 +67,10 @@ test_that("as_box_if() ensures boxed value if predicate returns TRUE", {
 
   expect_null(as_box_if(NULL, is_vector, "null_box"))
 
-  expect_error(as_box_if(NULL, ~ 10), "Predicate functions must return a single")
+  expect_snapshot({
+    (expect_error(as_box_if(NULL, ~ 10)))
+    (expect_error(as_box_if(NULL, ~ c(TRUE, FALSE))))
+  })
 })
 
 test_that("unboxing a non-boxed value is an error", {
@@ -77,7 +78,7 @@ test_that("unboxing a non-boxed value is an error", {
 })
 
 test_that("zap() creates a zap", {
-  expect_is(zap(), "rlang_zap")
+  expect_s3_class(zap(), "rlang_zap")
   expect_true(is_zap(zap()))
 })
 
@@ -98,7 +99,7 @@ test_that("done() can be empty", {
   expect_identical(unbox(empty), missing_arg())
 
   expect_true(is_done_box(empty))
-  expect_is(empty, "rlang_box_done")
+  expect_s3_class(empty, "rlang_box_done")
   expect_identical(empty %@% empty, TRUE)
 
   expect_true(is_done_box(empty, empty = TRUE))
